import time
import __main__
from traitlets.config.configurable import SingletonConfigurable


__main__._camera_flip_method = "2"

# JupyterNotebook Checker
def isnotebook():
    try:
        shell = get_ipython().__class__.__name__
        if shell == "ZMQInteractiveShell":
            return True
        else:
            return False
    except NameError:
        return False


__main__.isnotebook = isnotebook()

# ----------------------------------------------------------------------------------------#


def bgr8_to_jpeg(value):
    global cv2
    import cv2

    return bytes(cv2.imencode(".jpg", value)[1])


class _camera(SingletonConfigurable):
    cap_mode_table = {
        0: (3264, 2464, 21),
        1: (3264, 1848, 28),
        2: (1920, 1080, 30),
        3: (1280, 720, 60),
        4: (1280, 720, 120),
    }

    import traitlets

    value = traitlets.Any()
    width = traitlets.Integer(default_value=224).tag(config=True)
    height = traitlets.Integer(default_value=224).tag(config=True)
    cap_mode = traitlets.Integer(default_value=0).tag(config=True)

    def __init__(self, *args, **kwargs):
        global cv2
        global np
        import cv2
        import os, atexit
        import numpy as np

        super(_camera, self).__init__(*args, **kwargs)

        os.system("echo soda | sudo -S systemctl restart nvargus-daemon")

        capture_width, capture_height, fps = self.cap_mode_table[self.cap_mode]
        gst_str = (
            "nvarguscamerasrc ! video/x-raw(memory:NVMM), width=%d, height=%d, format=(string)NV12, framerate=(fraction)%d/1 ! nvvidconv flip-method=%s ! video/x-raw, width=(int)%d, height=(int)%d, format=(string)BGRx ! videoconvert ! appsink"
            % (
                capture_width,
                capture_height,
                fps,
                __main__._camera_flip_method,
                self.width,
                self.height,
            )
        )

        self.value = np.empty((self.height, self.width, 3), dtype=np.uint8)
        try:
            self.cap = cv2.VideoCapture(gst_str, cv2.CAP_GSTREAMER)

            success, _ = self.cap.read()
            if not success:
                raise RuntimeError("Could not read image from camera.")

            self.start()
        except:
            raise RuntimeError("Could not initialize camera.  Please see error trace.")

        atexit.register(self.stop)

    def __del__(self):
        self.stop()
        self.cap.release()

    def _capture_frames(self):
        while not self._is_stop:
            success, self.value = self.cap.read()
            if not success:
                raise RuntimeError("Could not read image from camera.")

    def start(self):
        global Thread
        from threading import Thread

        if not self.cap.isOpened():
            capture_width, capture_height, fps = self.cap_mode_table[self.cap_mode]
            gst_str = (
                "nvarguscamerasrc ! video/x-raw(memory:NVMM), width=%d, height=%d, format=(string)NV12, framerate=(fraction)%d/1 ! nvvidconv flip-method=%s ! video/x-raw, width=(int)%d, height=(int)%d, format=(string)BGRx ! videoconvert ! appsink"
                % (
                    capture_width,
                    capture_height,
                    fps,
                    __main__._camera_flip_method,
                    self.width,
                    self.height,
                )
            )
            self.cap = cv2.VideoCapture(gst_str, cv2.CAP_GSTREAMER)

        if not hasattr(self, "_thread") or not self._thread.isAlive():
            self._is_stop = False
            self._thread = Thread(target=self._capture_frames)
            self._thread.start()

    def stop(self):
        if hasattr(self, "_thread"):
            self._is_stop = True
            self._thread.join()
            self.value = np.empty((self.height, self.width, 3), dtype=np.uint8)

    def restart(self):
        self.stop()
        self.start()


# ----------------------------------------------------------------------------------------#


class IPyCamera:
    def __init__(self, width=224, height=224, cap_mode=0, auto_load=True):
        self.width = width
        self.height = height
        self.cap_mode = cap_mode

        if auto_load:
            self.__load()

    def __del__(self):
        self.stop()
        self.destroy()

    def __call__(self):
        return self.__image

    def __load(self):
        import ipywidgets.widgets as widgets
        import traitlets

        self.__camera = _camera.instance(
            width=self.width, height=self.height, cap_mode=self.cap_mode
        )
        self.__image = widgets.Image(
            format="jpeg", width=self.width, height=self.height
        )
        self.__camera_link = traitlets.dlink(
            (self.__camera, "value"), (self.__image, "value"), transform=bgr8_to_jpeg
        )

    def show(self):
        from IPython.display import display
        import traitlets

        if self.__camera is None:
            self.__load()

        if self.__camera_link is None:
            self.__camera_link = traitlets.dlink(
                (self.__camera, "value"),
                (self.__image, "value"),
                transform=bgr8_to_jpeg,
            )
            display(self.__image)
        else:
            display(self.__image)
            
    def destroy(self):
        self.__camera.stop()
        self.__camera.cap.release()

    def stop(self):
        if self.__camera_link is not None:
            self.__camera.stop()
            self.__camera_link.unlink()

    @property
    def value(self):
        return self.__camera.value


class Camera:
    def __init__(self, width=224, height=224, cap_mode=0):
        self.width = width
        self.height = height
        self.cap_mode = cap_mode

        self.__frame = None
        self.__value = None
        self.__thread = None
        self.__loop = True
        self.__running = False
        self.__load()

    def __del__(self):
        self.__loop = False
        if self.__thread is not None:
            self.__thread.join()
            self.__thread = None

    def __load(self):
        self.__camera = _camera.instance(
            width=self.width, height=self.height, cap_mode=self.cap_mode
        )
        self.__camera.restart()

    def __display(self):
        while self.__loop:
            if self.__running:
                if self.__value is not None:
                    cv2.imshow(self.__frame, self.__value)
                else:
                    cv2.imshow(self.__frame, self.__camera.value)

                if cv2.waitKey(1) & 0xFF == ord("q"):
                    self.stop()

    def stop(self):
        self.__camera.stop()
        self.__camera.cap.release()
        self.__camera = None
        self.__running = False
        cv2.destroyAllWindows()

    def show(self, frame="frame", value=None):
        if self.__camera is None:
            self.__load()

        self.__frame = frame
        self.__value = value
        self.__running = True

        if self.__thread is None:
            global Thread
            from threading import Thread

            self.__thread = Thread(target=self.__display, daemon=True)
            self.__thread.start()

    @property
    def value(self):
        return self.__camera.value


# ----------------------------------------------------------------------------------------#


class Audio:
    def __init__(self, blocking=True, cont=False):
        global pyaudio, wave
        import pyaudio, wave

        self.W = None
        self.p = pyaudio.PyAudio()
        self.stream = None
        self.blocking = blocking
        self.cont = cont
        self.isStop = None

    def __del__(self):
        self.close()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    def stop(self):
        self.isStop = True

    def close(self):
        self.stop()
        time.sleep(0.1)

        if self.W != None:
            if type(self.W) is list:
                for w in self.W:
                    w.close()
            else:
                self.W.close()

        if self.stream != None:
            self.stream.stop_stream()
            self.stream.close()

        self.p.terminate()


def audio_play(file):
    global wave
    global pyaudio
    import wave
    import pyaudio

    w = wave.open(file, "rb")
    data = w.readframes(w.getnframes())
    w.close()

    p = pyaudio.PyAudio()
    s = p.open(
        format=p.get_format_from_width(w.getsampwidth()),
        channels=w.getnchannels(),
        rate=w.getframerate(),
        output=True,
    )
    s.write(data)
    s.stop_stream()
    s.close()
    p.terminate()


# ----------------------------------------------------------------------------------------#


class AudioPlay(Audio):
    def __init__(self, file, blocking=True, cont=False):
        super().__init__(blocking, cont)

        self.W = wave.open(file, "rb")
        self.stream = self.p.open(
            format=self.p.get_format_from_width(self.W.getsampwidth()),
            channels=self.W.getnchannels(),
            rate=self.W.getframerate(),
            output=True,
            stream_callback=None if blocking else self._callback,
        )

        if blocking:
            self.data = self.W.readframes(self.W.getnframes())

    def __del__(self):
        super().__del__()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    def _callback(self, in_data, frame_count, time_info, status):
        data = self.W.readframes(frame_count)
        if self.cont:
            mod = frame_count - len(data) // self.W.getsampwidth()
            if mod != 0:
                self.W.rewind()
                data += self.W.readframes(mod)

        return (data, pyaudio.paContinue if not self.isStop else pyaudio.paAbort)

    def run(self):
        self.isStop = False
        if self.blocking:
            self.stream.write(self.data)
        else:
            self.stream.start_stream()

    def isPlay(self):
        return self.stream.is_active()


# ----------------------------------------------------------------------------------------#


class AudioPlayList(Audio):
    def __init__(self, files, blocking=True, cont=False):
        super().__init__(blocking, cont)

        self.W = []
        for file in files:
            self.W.append(wave.open(file, "rb"))

        self.stream = self.p.open(
            format=self.p.get_format_from_width(self.W[0].getsampwidth()),
            channels=self.W[0].getnchannels(),
            rate=self.W[0].getframerate(),
            output=True,
            stream_callback=None if blocking else self._callback,
        )

        self.data = []
        if blocking:
            for w in self.W:
                self.data.append(w.readframes(w.getnframes()))

    def _callback(self, in_data, frame_count, time_info, status):
        data = self.W[self.pos].readframes(frame_count)
        if self.cont:
            mod = frame_count - len(data) // self.W[self.pos].getsampwidth()
            if mod != 0:
                self.pos += 1
                if self.pos >= len(self.W):
                    self.pos = 0

                self.W[self.pos].rewind()
                data += self.W[self.pos].readframes(mod)

        return (data, pyaudio.paContinue if not self.isStop else pyaudio.paAbort)

    def run(self, pos=0):
        self.isStop = False
        self.pos = pos

        if self.blocking:
            self.stream.write(self.data[pos])
        else:
            self.stream.start_stream()

    def isPlay(self):
        return self.stream.is_active()


# ----------------------------------------------------------------------------------------#


class AudioRecord(Audio):
    def __init__(self, file, sFormat=8, sChannel=1, sRate=48000, sFramePerBuffer=1024):
        super().__init__(False)

        self.w = wave.open(file, "wb")
        self.w.setsampwidth(self.p.get_sample_size(sFormat))
        self.w.setnchannels(sChannel)
        self.w.setframerate(sRate)

        self.stream = self.p.open(
            format=sFormat,
            channels=sChannel,
            rate=sRate,
            input=True,
            frames_per_buffer=sFramePerBuffer,
            stream_callback=self._callback,
        )

    def __del__(self):
        super().__del__()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    def _callback(self, in_data, frame_count, time_info, status):
        self.w.writeframes(in_data)
        data = chr(0) * len(in_data)

        return (data, pyaudio.paContinue if not self.isStop else pyaudio.paAbort)

    def run(self):
        self.stream.start_stream()


# ----------------------------------------------------------------------------------------#


class Tone:
    def __init__(self, tempo=100, volume=0.5, rate=48000, channels=1):
        global pyaudio
        import pyaudio

        self.tempo = tempo
        self.volume = volume
        self.rate = rate
        self.channels = channels
        self.p = pyaudio.PyAudio()
        self.stream = self.p.open(
            format=pyaudio.paFloat32,
            channels=self.channels,
            rate=self.rate,
            output=True,
        )

    def __del__(self):
        self.close()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    def close(self):
        self.stream.stop_stream()
        self.stream.close()
        self.p.terminate()

    def setTempo(self, tempo):
        self.tempo = tempo

    def rest(self, duration):
        self.play(0, "REST", 1 / 4)

    def play(self, octave, pitch, duration):
        global np
        import numpy as np

        """
        octave = 1 ~ 8
        note = DO, RE, MI, FA, SOL, RA, SI
        dulation = 1, 1/2, 1/4, 1/8, 1/16, 1/32, 1/64, ...
        """
        string_to_pitch = {
            "REST": 0,
            "DO": 1,
            "DO#": 2,
            "RE": 3,
            "RE#": 4,
            "MI": 5,
            "FA": 6,
            "FA#": 7,
            "SOL": 8,
            "SOL#": 9,
            "RA": 10,
            "RA#": 11,
            "SI": 12,
        }

        p = string_to_pitch[pitch]
        f = 2 ** (octave) * 55 * 2 ** ((p - 10) / 12)

        if p == 0:
            time.sleep((60.0 / self.tempo) * (duration * 4))
        else:
            sample = (
                np.sin(
                    2
                    * np.pi
                    * np.arange(self.rate * (60.0 / self.tempo * 4) * (duration * 4))
                    * f
                    / self.rate
                )
            ).astype(np.float32)
            self.stream.write(self.volume * sample)

            time.sleep(0.02)


# ----------------------------------------------------------------------------------------#


class SoundMeter:
    global pyaudio
    import pyaudio

    def __init__(
        self,
        sampleFormat=pyaudio.paInt16,
        channelNums=1,
        framesPerBuffer=1024,
        sampleRate=48000,
    ):
        self.func = None
        self.args = None
        self.isStop = False

        self.p = pyaudio.PyAudio()

        self.stream = self.p.open(
            format=sampleFormat,
            channels=channelNums,
            rate=sampleRate,
            input=True,
            frames_per_buffer=framesPerBuffer,
            stream_callback=self._callback,
        )

    def __del__(self):
        self.stop()

    def setCallback(self, func, *args):
        self.func = func
        self.args = args
        self.stream.start_stream()

    def stop(self):
        self.isStop = True
        self.stream.stop_stream()
        self.stream.close()
        self.p.terminate()

    def _callback(self, inData, frameCount, timeInfo, status):
        import audioop

        rms = audioop.rms(inData, 2)
        self.func(rms, inData, *self.args)

        data = chr(0) * len(inData)
        return (data, pyaudio.paContinue if not self.isStop else pyaudio.paAbort)



from .CAN import Can
import threading


class RGB:
    ACT_RGB = 0x250

    def __init__(self, dev="can0", bitrate=500000):
        self.__can = Can(dev, bitrate)

    def __del__(self):
        self.stop()

    def on(self, r=255, g=255, b=255):
        self.__can.write(RGB.ACT_RGB, [0x0F, r, g, b])

    def off(self):
        self.__can.write(RGB.ACT_RGB, [0x0F, 0, 0, 0])

    def stop(self):        
        self.__can.write(RGB.ACT_RGB, [0x0F, 0, 0, 0])



from .lidar2d_imp import (
    _object,
    _swig_setattr,
    _swig_getattr,
    _swig_repr,
    _lidar2d,
    __builtin__,
)
import numpy as np


class Rplidar(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Rplidar, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Rplidar, name)
    __repr__ = _swig_repr

    BAUDRATE_A2 = 256_000
    BAUDRATE_S2 = 1000_000

    MODE_STANDARD = 0  # 4KHz
    MODE_S2_BOOST = 1  # 32KHz
    MODE_A2_BOOST = 2  # 8KHz

    PORT = "/dev/ttyUSB0"

    def __init__(self, port=PORT, baudrate=BAUDRATE_S2):
        this = _lidar2d.new_Lidar2D()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
        assert self.connect(port, baudrate) and "Unknown serial..."

    __swig_destroy__ = _lidar2d.delete_Lidar2D
    __del__ = lambda self: None

    def connect(self, port=PORT, baudrate=BAUDRATE_A2):
        return _lidar2d.Lidar2D_connect(self, port, baudrate)

    def startMotor(self, mode=MODE_S2_BOOST):
        return _lidar2d.Lidar2D_startScan(self, mode)

    def stopMotor(self):
        return _lidar2d.Lidar2D_stopScan(self)

    def getVectors(self, limit_distance=0, degree=True, quality=True):
        ret = np.array(_lidar2d.Lidar2D_getData(self, limit_distance, degree, quality))
        return ret

    def getXY(self, limit_distance=0, quality=True):
        return np.array(_lidar2d.Lidar2D_getXY(self, limit_distance, quality))[:, ::-1]

    def getMap(self, size=(300, 300), limit_distance=12000, quality=True):
        pixmap = np.zeros(size[::-1])

        width, height = (None, None)

        if type(size) in (tuple, list):
            width, height = (size[0] / 2, size[1] / 2)
        elif type(size) in (str, int, float):
            size = int(size)
            width, height = (size / 2, size / 2)
        else:
            raise (ValueError("The argument 'size' is not available."))

        coords = np.array(self.getXY(limit_distance=limit_distance, quality=quality))

        is_ranged = np.mean(np.abs(coords) < limit_distance, axis=1) == 1.0

        for i in range(len(is_ranged)):
            if is_ranged[i]:
                x, y = ((coords[i] * (1.0, -1.0)) / limit_distance) * (
                    width,
                    height,
                ) + (width, height)
                pixmap[int(y), int(x)] += 1

        std = np.mean(pixmap) + np.std(pixmap)

        if std != 0:
            pixmap = np.where(pixmap > std, std, pixmap)
            pixmap = (pixmap / std) * 255

        # pixmap = np.rot90(pixmap)
        return pixmap


Lidar2D_swigregister = _lidar2d.Lidar2D_swigregister
Lidar2D_swigregister(Rplidar)



import time
import struct
import threading
import numpy as np

from .config import IP, GROUP, __version__
from .Util import ICAN

class Constants:
    US = 1e-6
    US_10 = 1e-5
    US_100 = 1e-4
    MS = 1e-3
    MS_10 = 1e-2
    MS_100 = 1e-1
    MS_1000 = 1.0


class KalmanFilter:
    def __init__(
        self,
        num_sensors,
        measurement_time=Constants.MS,
        initial_state=None,
        initial_covariance=None,
    ):
        self.num_sensors = num_sensors
        self.dt = measurement_time

        self.x = (
            initial_state if initial_state is not None else np.zeros((num_sensors, 1))
        )

        self.P = (
            initial_covariance
            if initial_covariance is not None
            else np.eye(num_sensors) * 100
        )

        self.F = np.eye(num_sensors)
        self.B = np.zeros((num_sensors, num_sensors))
        self.u = np.zeros((num_sensors, 1))
        self.H = np.eye(num_sensors)
        self.Q = np.eye(num_sensors) * 0.1
        self.R = np.eye(num_sensors) * 3

    def predict(self):
        self.x = np.dot(self.F, self.x) + np.dot(self.B, self.u)
        self.P = np.dot(np.dot(self.F, self.P), self.F.T) + self.Q

    def update(self, z):
        K = np.dot(
            np.dot(self.P, self.H.T),
            np.linalg.inv(np.dot(np.dot(self.H, self.P), self.H.T) + self.R),
        )
        self.x = self.x + np.dot(K, (z - np.dot(self.H, self.x)))
        self.P = self.P - np.dot(np.dot(K, self.H), self.P)


class ExponentialSmoothingFilter:
    def __init__(self, window_size=5, alpha=0.15):
        self.__window_size = window_size
        self.__alpha = alpha
        self.__data_list = []

    def update(self, data):
        self.__data_list.append(data)
        if len(self.__data_list) > self.__window_size:
            self.__data_list.pop(0)
        if len(self.__data_list) == 1:
            filtered_value = data
        else:
            filtered_value = (
                self.__alpha * data + (1 - self.__alpha) * self.__data_list[-2]
            )

        return filtered_value
    
    
class IGroup(ICAN):
    ican_groups = {}
    
    def __init__(self, ip, group):
        if group not in IGroup.ican_groups:
            super().__init__(ip, group)
            IGroup.ican_groups[group] = self.ican
        else:
            self.ican = IGroup.ican_groups[group]            
            if ip is not IP:
                self.ican.appendRemote(ip)
            
    def __del__(self):
        pass
    

class IMU:
    class Sensor(IGroup):
        def __init__(self, ip=IP, group=GROUP):
            IMU_ID = self.__class__.ID
            IMU_GPORT = self.__class__.GPORT
            
            super().__init__(ip, group)
            self.__ican = self.ican_groups[group]
            self.__ican.addMcastReceiver(
                IMU_GPORT,
                IMU_ID,
            )

        def __del__(self):
            pass

        def __unpack(self, data):
            if self.ID == IMU.Accel.ID:
                data_scale = struct.unpack("hhh", struct.pack("6B", *data))
                data = tuple(map(lambda n: n / 100, data_scale[:]))
                return data
            elif self.ID == IMU.Gravity.ID:
                data_scale = struct.unpack("hhh", struct.pack("6B", *data))
                data = tuple(map(lambda n: n / 100, data_scale[:]))
                return data
            elif self.ID == IMU.Gyro.ID:
                data_scale = struct.unpack("hhh", struct.pack("6B", *data))
                data = tuple(map(lambda n: n / 916, data_scale[:]))
                return data
            elif self.ID == IMU.Magnetic.ID:
                data_scale = struct.unpack("hhh", struct.pack("6B", *data))
                data = tuple(map(lambda n: n / 16, data_scale[:]))
                return data
            elif self.ID == IMU.Euler.ID:
                data_scale = struct.unpack("hhh", struct.pack("6B", *data))
                data = tuple(map(lambda n: n / 16, data_scale[:]))
                return data
            elif self.ID == IMU.Quat.ID:
                data_scale = struct.unpack("hhhh", struct.pack("8B", *data))
                data = tuple(map(lambda n: n / (1 << 14), data_scale[:]))
                return data

        @property
        def id(self):
            return self.ID

        @property
        def gport(self):
            return self.GPORT

        def decode(self, raw):
            unpacked_value = self.__unpack(raw)
            return unpacked_value

    class Accel(Sensor):
        ID = 0x121
        GPORT = 3208

    class Gravity(Sensor):
        ID = 0x125
        GPORT = 3209

    class Gyro(Sensor):
        ID = 0x123
        GPORT = 3210

    class Magnetic(Sensor):
        ID = 0x122
        GPORT = 3211

    class Euler(Sensor):
        ID = 0x124
        GPORT = 3212

    class Quat(Sensor):
        ID = 0x126
        GPORT = 3213
        

class Ultrasonic(IGroup):
    ID_ULTRASONIC = 0x130
    GPORT_ULTRASONIC = 3202

    NUM_SENSORS = 2
    MIN_DISTANCE = 10
    MAX_DISTANCE = 180

    def __init__(self, ip=IP, group=GROUP):
        super().__init__(ip, group)
        self.__ican = self.ican_groups[group]
        self.__ican.addMcastReceiver(
            Ultrasonic.GPORT_ULTRASONIC,
            Ultrasonic.ID_ULTRASONIC,
        )
        self.__kf = KalmanFilter(self.NUM_SENSORS)

    def __del__(self):
        pass

    @property
    def id(self):
        return self.ID_ULTRASONIC

    @property
    def gport(self):
        return self.GPORT_ULTRASONIC

    def decode(self, raw):
        z = np.array(raw).reshape(-1, 1)
        self.__kf.predict()
        self.__kf.update(z)
        filtered_distances = self.__kf.x.flatten()

        return filtered_distances

class Encoder(IGroup):
    ID_ENCODER_RST = 0x012
    ID_ENCODER = 0x111
    GPORT_ENCODER = 3203

    ENCODER_0 = 0x01
    ENCODER_1 = 0x02

    def __init__(self, ip=IP, group=GROUP):
        super().__init__(ip, group)
        self.__ican = self.ican_groups[group]
        self.__ican.addMcastReceiver(
            Encoder.GPORT_ENCODER,
            Encoder.ID_ENCODER,
        )
        self.reset()

    @property
    def id(self):
        return self.ID_ENCODER

    @property
    def gport(self):
        return self.GPORT_ENCODER

    def reset(self, value=(ENCODER_0 | ENCODER_1)):
        self.__ican.write(self.ID_ENCODER_RST, [value & 0x07])

    def decode(self, raw):
        return (((raw[1] & 0xFF) << 8) | raw[0])


class Cds(IGroup):
    ID_CDS = 0x1B0
    GPORT_CDS = 3204

    def __init__(self, ip=IP, group=GROUP, window_size=7, alpha=0.15):
        super().__init__(ip, group)
        self.__ican = self.ican_groups[group]
        self.__ican.addMcastReceiver(
            Cds.GPORT_CDS,
            Cds.ID_CDS,
        )
        self.__smf = ExponentialSmoothingFilter()

    @property
    def id(self):
        return self.ID_CDS

    @property
    def gport(self):
        return self.GPORT_CDS

    def decode(self, raw):
        data = (((raw[1] & 0xFF) << 8) | raw[0])
        filtered_value = self.__smf.update(data)

        return filtered_value


class Battery(IGroup):
    ID_BATTERY = 0x1A0
    GPORT_BATTERY = 3205

    def __init__(self, ip=IP, group=GROUP):
        super().__init__(ip, group)
        self.__ican = self.ican_groups[group]
        self.__ican.addMcastReceiver(
            Battery.GPORT_BATTERY,
            Battery.ID_BATTERY,
        )

    @property
    def id(self):
        return self.ID_BATTERY

    @property
    def gport(self):
        return self.GPORT_BATTERY

    def decode(self, raw):
        return round((((raw[1] & 0x0F) << 8) | raw[0]) / 10, 1)



import warnings


__version__ = 0.9

IP = '127.0.0.1'
GROUP = "238.2.3.2"
try:
    with open("/etc/product", "r") as file:
        for line in file:
            key, value = line.split("=", 1)
            if key == "MCAST_GROUP":
                GROUP = value.strip()
                break
except FileNotFoundError:
    def formatwarning(message, category, *args, **kwargs):
        return f"{category.__name__}: [W] {message}\n"
    warnings.formatwarning = formatwarning
    warnings.warn("Remote mode is activated. Please, set your ip and multicast group properly to use!", category=UserWarning)
    
    



import math
import numpy as np

from .config import IP, GROUP, __version__
from .Util import ICAN

class Driving(ICAN):
    ID_DRIVING = 0x010
    THROTTLE_0POINT = 100
    WHEEL_STOP = 0
    WHEEL_FORWARD = 1
    WHEEL_BACKWARD = 2
    STEER_ID = 0x013
    STEER_CENTER = 90

    def __init__(self, ip=IP, group=GROUP):
        super().__init__(ip, group)
        self.__ican = self.ican
        self.__steering = 0.0
        self.__throttle = 0
        self.stop()

    def __transfer(self, id, payload, is_extended=False):
        self.__ican.write(
            id, payload, is_extended
        )

    def move(self, throttle=None, cm=None, direction=None):
        if throttle is not None:
            self.__throttle = throttle
            
        if self.__moving != direction:
            self.__moving = direction
        
        if self.__moving == self.WHEEL_FORWARD:
            if self.__throttle < 0:
                self.__throttle *= -1
        elif self.__moving == self.WHEEL_BACKWARD:
            if self.__throttle > 0:
                self.__throttle *= -1
        
        if cm is None:
            payload = [100+self.__throttle, 0]
        else:
            payload = [100+self.__throttle, cm]
        self.__transfer(self.ID_DRIVING, payload)

    def stop(self):
        self.__throttle = 0
        self.__moving = self.WHEEL_STOP
        self.__transfer(self.ID_DRIVING, [100+self.__throttle,0])

    @property
    def throttle(self):
        return self.__throttle

    @throttle.setter
    def throttle(self, throttle):
        if 0 <= throttle <= 100:
            if self.__moving == self.WHEEL_BACKWARD:
                self.__throttle = throttle * -1
            else:
                self.__throttle = throttle
        
            if self.__moving is not self.WHEEL_STOP:
                self.move()
        else:
            print("Warning : This value is out of range 0 to 100")

    @property
    def steering(self):
        return self.__steering

    @steering.setter
    def steering(self, value):
        if value > 1.0 or value < -1.0:
            print("Warning : This value is out of range -1.0 to 1.0")
            if value > 1.0:
                value = 1.0
            elif value < -1.0:
                value = -1.0
        self.__steering = value 
        data = self.STEER_CENTER + int(value*35)
        self.__transfer(self.STEER_ID,[data&0xff,(data>>8)&0xff])

    def forward(self, throttle=None):
        self.move(throttle,0,self.WHEEL_FORWARD)

    def backward(self, throttle=None):
        self.move(throttle,0,self.WHEEL_BACKWARD)



from .CAN import Can
import threading


class PGCA:
    INIT_PGCA = 0x290
    ACT_PGCA = 0x291
    BRD_PGCA_P = 0x390
    BRD_PGCA_G = 0x391
    BRD_PGCA_C = 0x392
    BRD_PGCA_A = 0x393

    def __init__(self, dev="can0", bitrate=500000):
        self.__can = Can(dev, bitrate)
        self.__func = None
        self.__param = None
        self.__thread = None
        self.__stop = False

        self.__pgca = {
            PGCA.BRD_PGCA_P: None,
            PGCA.BRD_PGCA_G: None,
            PGCA.BRD_PGCA_C: None,
            PGCA.BRD_PGCA_A: None,
        }
        for f in self.__pgca.keys():
            self.__can.setFilter(f)
        self.__can.write(PGCA.INIT_PGCA, [0xFF])

    def __del__(self):
        self.stop()

    # def __callback(self):
    #     while not self.__stop:
    #         id, dlc, payload = self.__can.read(timeout=0.1)

    #         if not payload:
    #             continue
    #         elif id == PGCA.BRD_PGCA_P:
    #             self.__pgca[id] = payload[0]
    #         elif id == PGCA.BRD_PGCA_G:
    #             if payload[0] == 1:
    #                 self.__pgca[id] = "Up"
    #             elif payload[0] == 2:
    #                 self.__pgca[id] = "Down"
    #             elif payload[0] == 4:
    #                 self.__pgca[id] = "Left"
    #             elif payload[0] == 8:
    #                 self.__pgca[id] = "Right"
    #         elif id == PGCA.BRD_PGCA_C:
    #             self.__pgca[id] = (
    #                 (payload[1] << 8) | payload[0],
    #                 (payload[3] << 8) | payload[2],
    #                 (payload[5] << 8) | payload[4],
    #                 (payload[7] << 8) | payload[6],
    #             )
    #         elif id == PGCA.BRD_PGCA_A:
    #             self.__pgca[id] = (payload[1] << 8) | payload[0]

    #         if self.__param:
    #             self.__func(tuple(self.__pgca.values()), self.__param)
    #         else:
    #             self.__func(tuple(self.__pgca.values()))

    def Proximity(self):
        try:
            self.__can.write(PGCA.ACT_PGCA, [0x01, 0x00])
            id, dlc, payload = self.__can.read(timeout=2.0)

            if id == PGCA.BRD_PGCA_P:
                self.__pgca[id] = payload[0]

            return self.__pgca[id]
        except:
            return None

    def Gesture(self):
        try:
            self.__can.write(PGCA.ACT_PGCA, [0x02, 0x00])
            id, dlc, payload = self.__can.read(timeout=2.0)

            if id == PGCA.BRD_PGCA_G:
                if payload[0] == 1:
                    self.__pgca[id] = "Up"
                elif payload[0] == 2:
                    self.__pgca[id] = "Down"
                elif payload[0] == 4:
                    self.__pgca[id] = "Left"
                elif payload[0] == 8:
                    self.__pgca[id] = "Right"
                else:
                    return None

            return self.__pgca[id]
        except:
            return None

    def Color(self):
        try:
            self.__can.write(PGCA.ACT_PGCA, [0x04, 0x00])
            id, dlc, payload = self.__can.read(timeout=2.0)

            if id == PGCA.BRD_PGCA_C:
                self.__pgca[id] = (
                    (payload[1] << 8) | payload[0],
                    (payload[3] << 8) | payload[2],
                    (payload[5] << 8) | payload[4],
                    (payload[7] << 8) | payload[6],
                )
                return self.__pgca[id]
            else:
                return None
        except:
            return None

    def ALS(self):
        try:
            self.__can.write(PGCA.ACT_PGCA, [0x08, 0x00])
            id, dlc, payload = self.__can.read(timeout=2.0)

            if id == PGCA.BRD_PGCA_A:
                self.__pgca[id] = (payload[1] << 8) | payload[0]
                return self.__pgca[id]
            else:
                return None
        except:
            return None

    # def callback(self, func, repeat=1, param=None):
    #     if not self.__thread:
    #         self.__func = func
    #         self.__param = param
    #         self.__stop = False
    #         self.__thread = threading.Thread(target=self.__callback)
    #         self.__thread.start()
    #         self.__can.write(PGCA.ACT_PGCA, [0x0F, repeat & 0xFF])

    def stop(self):
        if self.__thread:
            self.__stop = True
            self.__thread = None
            self.__can.write(PGCA.ACT_PGCA, [0x0, 0x00])



from .CAN import Can
import threading


class Analog:
    ACT_Analog = 0x260
    BRD_Analog = 0x360

    def __init__(self, dev="can0", bitrate=500000):
        self.__can = Can(dev, bitrate)
        self.__func = None
        self.__param = None
        self.__thread = None
        self.__stop = False
        self.__can.setFilter(Analog.BRD_Analog)

    def __del__(self):
        self.stop()

    def __callback(self):
        cds, ntc, vr = 0, 0, 0

        while not self.__stop:
            id, dlc, payload = self.__can.read(timeout=0.1)

            if not payload:
                continue
            elif id == Analog.BRD_Analog:
                cds = (payload[1] << 8) | payload[0]
                ntc = (payload[3] << 8) | payload[2]
                vr = (payload[5] << 8) | payload[4]

            if self.__param:
                self.__func((cds, ntc, vr), self.__param)
            else:
                self.__func((cds, ntc, vr))

    def read(self):
        try:
            self.__can.write(Analog.ACT_Analog, [0x1F, 0x00])
            id, dlc, payload = self.__can.read(timeout=2.0)

            if id == Analog.BRD_Analog:
                cds = (payload[1] << 8) | payload[0]
                ntc = (payload[3] << 8) | payload[2]
                vr = (payload[5] << 8) | payload[4]

            return cds, ntc, vr
        except:
            return None

    def callback(self, func, repeat=1, param=None):
        if not self.__thread:
            self.__func = func
            self.__param = param
            self.__stop = False
            self.__thread = threading.Thread(target=self.__callback)
            self.__thread.start()
            self.__can.write(Analog.ACT_Analog, [0x1F, repeat & 0xFF])

    def stop(self):
        if self.__thread:
            self.__stop = True
            self.__thread = None
            self.__can.write(Analog.ACT_Analog, [0x0, 0x00])



import cv2
import librosa
import math
import time
import threading
import ipywidgets as widgets
import numpy as np
from IPython.display import display

from genlib.ican import InetCAN
from .config import __version__

def _image_resize(image, width=None, height=None, inter=cv2.INTER_AREA):
    dim = None
    (h, w) = image.shape[:2]

    if width is None and height is None:
        return image

    if width is None:
        r = height / float(h)
        dim = (int(w * r), height)
    else:
        r = width / float(w)
        dim = (width, int(h * r))

    resized = cv2.resize(image, dim, interpolation=inter)

    return resized


windows = {}


def imshow(title, image, width=300, height=300, mode="BGR"):
    import __main__

    global windows
    if not title in windows:
        windows[title] = {
            "widget": widgets.Image(width=width, height=height),
            "kernel": [],
        }
    elif (
        windows[title]["widget"].width != width
        or windows[title]["widget"].height != height
    ):
        if windows[title]["widget"].width != width:
            windows[title]["widget"].width = width

        if windows[title]["widget"].height != height:
            windows[title]["widget"].height = height

    _image = windows[title]["widget"]

    h, w = image.shape[:2]
    ih = int(_image.height)
    iw = int(_image.width)

    w_ratio = abs(h / w - ih / iw)
    h_ratio = abs(w / h - ih / iw)

    if w_ratio <= h_ratio:
        img = _image_resize(image, width=iw)
    else:
        img = _image_resize(image, height=ih)

    if mode.lower == "rgb":
        img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)

    img = bytes(cv2.imencode(".jpg", img)[1])
    _image.value = img

    kernel_num = len(__main__.In) - 1

    if not kernel_num in windows[title]["kernel"]:
        windows[title]["kernel"].append(kernel_num)
        display(_image)


def enable_imshow():
    import __main__

    if "get_ipython" in dir(__main__):
        if not ("cv2" in dir(__main__)):
            import cv2

            __main__.cv2 = cv2

        __main__.cv2.imshow = imshow


def toMFCC(file_path, duration=1.0, rate=8000):
    wave, sr = librosa.load(file_path, mono=True, sr=rate)

    padlen = math.ceil(duration * (sr / int(0.048 * sr)))

    wave = librosa.util.normalize(wave)
    mfcc = librosa.feature.mfcc(
        wave, sr=sr, n_mfcc=40, hop_length=int(0.048 * sr), n_fft=int(0.096 * sr)
    )
    mfcc -= np.mean(mfcc, axis=0) + 1e-8

    pad_width = padlen - mfcc.shape[1]
    if pad_width > 0:
        mfcc = np.pad(mfcc, pad_width=((0, 0), (0, pad_width)), mode="constant")
    elif pad_width < 0:
        mfcc = mfcc[:, :padlen]

    return np.expand_dims(mfcc, -1)


def gstrmer(width=640, height=480, fps=30, flip=0):
    capture_width = width
    capture_height = height
    display_width = width
    display_height = height
    framerate = fps
    flip_method = flip

    return (
        "nvarguscamerasrc ! "
        "video/x-raw(memory:NVMM), "
        "width=(int)%d, height=(int)%d, "
        "format=(string)NV12, framerate=(fraction)%d/1 ! "
        "nvvidconv flip-method=%s ! "
        "video/x-raw, width=(int)%d, height=(int)%d, format=(string)BGRx ! "
        "videoconvert ! "
        "video/x-raw, format=(string)BGR ! appsink drop=1"
        % (
            capture_width,
            capture_height,
            framerate,
            flip_method,
            display_width,
            display_height,
        )
    )
    

class Constants:
    US = 1e-6
    US_10 = 1e-5
    US_100 = 1e-4
    MS = 1e-3
    MS_10 = 1e-2
    MS_100 = 1e-1
    MS_1000 = 1.0


class ICAN:
    def __init__(self, ip, group):
        self.ican = InetCAN(ip, group)
        self.__thread = None

    def __del__(self):
        if self.__thread:
            self.__running = False
            self.__thread.join()

    def __on_read(self):
        while self.__running:
            arbitration_id, data = self.read(timeout=Constants.MS_100)
            if arbitration_id == self.id:
                data = self.decode(data)
                if isinstance(data, np.ndarray):
                    data = data.tolist()
                self.__callback(data, *self.args, **self.kwargs)

    @property
    def id(self):
        raise NotImplementedError()

    @property
    def gport(self):
        raise NotImplementedError()

    def decode(self, raw):
        raise NotImplementedError()
    
    def read(self, gport=None, timeout=Constants.MS_100):
        start_time = time.time()
        self.ican.emptyBuffer(gport or self.gport)
        while timeout is None or time.time() - start_time < timeout: 
            arbitration_id, data = self.ican.read(gport or self.gport) # need to empty the buffer.
            if arbitration_id == self.id:
                data = self.decode(data)
                if isinstance(data, np.ndarray):
                    data = data.tolist()
                return data

        return None

    def register_callback(self, callback, *args, **kwargs):
        self.__callback = callback
        self.__args = args
        self.__kwargs = kwargs

        self.__running = True
        self.__thread = threading.Thread(target=self.__on_read, daemon=True)
        self.__thread.start()
        



from .CAN import Can
import threading


class TPHG:
    INIT_TPHG = 0x280
    ACT_TPHG = 0x281
    BRD_TPHG = 0x380

    def __init__(self, dev="can0", bitrate=500000):
        self.__can = Can(dev, bitrate)
        self.__func = None
        self.__param = None
        self.__thread = None
        self.__stop = False
        self.__can.setFilter(TPHG.BRD_TPHG)
        self.__can.write(TPHG.INIT_TPHG, [0xFF])

    def __del__(self):
        self.stop()

    def __callback(self):
        while not self.__stop:
            id, dlc, payload = self.__can.read(timeout=0.1)
            if not payload:
                continue
            elif id == TPHG.BRD_TPHG:
                temp = payload[0]
                press = (payload[2] << 8) | payload[1]
                humi = payload[3]
                gas = (
                    (payload[7] << 24)
                    | (payload[6] << 16)
                    | (payload[5] << 8)
                    | payload[4]
                )

                if self.__param:
                    self.__func((temp, press, humi, gas), self.__param)
                else:
                    self.__func((temp, press, humi, gas))

    def read(self):
        try:
            self.__can.write(TPHG.ACT_TPHG, [0x0f,0x00])
            id, dlc, payload = self.__can.read(timeout=2.0)

            if id == TPHG.BRD_TPHG:
                temp = payload[0]
                press = (payload[2] << 8) | payload[1]
                humi = payload[3]
                gas = (
                    (payload[7] << 24)
                    | (payload[6] << 16)
                    | (payload[5] << 8)
                    | payload[4]
                )

            return temp, press, humi, gas
        except:
            return None

    def callback(self, func, repeat=1, param=None):
        if not self.__thread:
            self.__func = func
            self.__param = param
            self.__stop = False
            self.__thread = threading.Thread(target=self.__callback)
            self.__thread.start()
            self.__can.write(TPHG.ACT_TPHG, [0x0f,repeat & 0xFF])

    def stop(self):
        if self.__thread:
            self.__stop = True
            self.__thread = None
            self.__can.write(TPHG.ACT_TPHG, [0x0, 0x00])



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.
import numpy as np

from sys import version_info as _swig_python_version_info
if _swig_python_version_info >= (2, 7, 0):
    def swig_import_helper():
        import importlib
        pkg = __name__.rpartition('.')[0]
        mname = '.'.join((pkg, '_rplidar')).lstrip('.')
        try:
            return importlib.import_module(mname)
        except ImportError:
            return importlib.import_module('_rplidar')
    _rplidar = swig_import_helper()
    del swig_import_helper
elif _swig_python_version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_rplidar', [dirname(__file__)])
        except ImportError:
            import _rplidar
            return _rplidar
        try:
            _mod = imp.load_module('_rplidar', fp, pathname, description)
        finally:
            if fp is not None:
                fp.close()
        return _mod
    _rplidar = swig_import_helper()
    del swig_import_helper
else:
    import _rplidar
del _swig_python_version_info

try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError("'%s' object has no attribute '%s'" % (class_type.__name__, name))


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except __builtin__.Exception:
    class _object:
        pass
    _newclass = 0

class SwigPyIterator(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SwigPyIterator, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SwigPyIterator, name)

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _rplidar.delete_SwigPyIterator
    __del__ = lambda self: None

    def value(self):
        return _rplidar.SwigPyIterator_value(self)

    def incr(self, n=1):
        return _rplidar.SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        return _rplidar.SwigPyIterator_decr(self, n)

    def distance(self, x):
        return _rplidar.SwigPyIterator_distance(self, x)

    def equal(self, x):
        return _rplidar.SwigPyIterator_equal(self, x)

    def copy(self):
        return _rplidar.SwigPyIterator_copy(self)

    def next(self):
        return _rplidar.SwigPyIterator_next(self)

    def __next__(self):
        return _rplidar.SwigPyIterator___next__(self)

    def previous(self):
        return _rplidar.SwigPyIterator_previous(self)

    def advance(self, n):
        return _rplidar.SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        return _rplidar.SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        return _rplidar.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        return _rplidar.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        return _rplidar.SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        return _rplidar.SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        return _rplidar.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self
SwigPyIterator_swigregister = _rplidar.SwigPyIterator_swigregister
SwigPyIterator_swigregister(SwigPyIterator)

class RplidarScanMode(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, RplidarScanMode, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, RplidarScanMode, name)
    __repr__ = _swig_repr
    __swig_setmethods__["id"] = _rplidar.RplidarScanMode_id_set
    __swig_getmethods__["id"] = _rplidar.RplidarScanMode_id_get
    if _newclass:
        id = _swig_property(_rplidar.RplidarScanMode_id_get, _rplidar.RplidarScanMode_id_set)
    __swig_setmethods__["us_per_sample"] = _rplidar.RplidarScanMode_us_per_sample_set
    __swig_getmethods__["us_per_sample"] = _rplidar.RplidarScanMode_us_per_sample_get
    if _newclass:
        us_per_sample = _swig_property(_rplidar.RplidarScanMode_us_per_sample_get, _rplidar.RplidarScanMode_us_per_sample_set)
    __swig_setmethods__["max_distance"] = _rplidar.RplidarScanMode_max_distance_set
    __swig_getmethods__["max_distance"] = _rplidar.RplidarScanMode_max_distance_get
    if _newclass:
        max_distance = _swig_property(_rplidar.RplidarScanMode_max_distance_get, _rplidar.RplidarScanMode_max_distance_set)
    __swig_setmethods__["ans_type"] = _rplidar.RplidarScanMode_ans_type_set
    __swig_getmethods__["ans_type"] = _rplidar.RplidarScanMode_ans_type_get
    if _newclass:
        ans_type = _swig_property(_rplidar.RplidarScanMode_ans_type_get, _rplidar.RplidarScanMode_ans_type_set)
    __swig_setmethods__["scan_mode"] = _rplidar.RplidarScanMode_scan_mode_set
    __swig_getmethods__["scan_mode"] = _rplidar.RplidarScanMode_scan_mode_get
    if _newclass:
        scan_mode = _swig_property(_rplidar.RplidarScanMode_scan_mode_get, _rplidar.RplidarScanMode_scan_mode_set)

    def __init__(self):
        this = _rplidar.new_RplidarScanMode()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _rplidar.delete_RplidarScanMode
    __del__ = lambda self: None
RplidarScanMode_swigregister = _rplidar.RplidarScanMode_swigregister
RplidarScanMode_swigregister(RplidarScanMode)

DRIVER_TYPE_SERIALPORT = _rplidar.DRIVER_TYPE_SERIALPORT
DRIVER_TYPE_TCP = _rplidar.DRIVER_TYPE_TCP
class ChannelDevice(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, ChannelDevice, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, ChannelDevice, name)

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def bind(self, arg2, arg3):
        return _rplidar.ChannelDevice_bind(self, arg2, arg3)

    def open(self):
        return _rplidar.ChannelDevice_open(self)

    def close(self):
        return _rplidar.ChannelDevice_close(self)

    def flush(self):
        return _rplidar.ChannelDevice_flush(self)

    def waitfordata(self, data_count, timeout=-1, returned_size=None):
        return _rplidar.ChannelDevice_waitfordata(self, data_count, timeout, returned_size)

    def senddata(self, data, size):
        return _rplidar.ChannelDevice_senddata(self, data, size)

    def recvdata(self, data, size):
        return _rplidar.ChannelDevice_recvdata(self, data, size)

    def setDTR(self):
        return _rplidar.ChannelDevice_setDTR(self)

    def clearDTR(self):
        return _rplidar.ChannelDevice_clearDTR(self)

    def ReleaseRxTx(self):
        return _rplidar.ChannelDevice_ReleaseRxTx(self)
    __swig_destroy__ = _rplidar.delete_ChannelDevice
    __del__ = lambda self: None
ChannelDevice_swigregister = _rplidar.ChannelDevice_swigregister
ChannelDevice_swigregister(ChannelDevice)

class RPlidarDriver(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, RPlidarDriver, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, RPlidarDriver, name)

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    DEFAULT_TIMEOUT = _rplidar.RPlidarDriver_DEFAULT_TIMEOUT
    MAX_SCAN_NODES = _rplidar.RPlidarDriver_MAX_SCAN_NODES
    LEGACY_SAMPLE_DURATION = _rplidar.RPlidarDriver_LEGACY_SAMPLE_DURATION
    if _newclass:
        CreateDriver = staticmethod(_rplidar.RPlidarDriver_CreateDriver)
    else:
        CreateDriver = _rplidar.RPlidarDriver_CreateDriver
    if _newclass:
        DisposeDriver = staticmethod(_rplidar.RPlidarDriver_DisposeDriver)
    else:
        DisposeDriver = _rplidar.RPlidarDriver_DisposeDriver

    def connect(self, arg2, arg3, flag=0):
        return _rplidar.RPlidarDriver_connect(self, arg2, arg3, flag)

    def disconnect(self):
        return _rplidar.RPlidarDriver_disconnect(self)

    def isConnected(self):
        return _rplidar.RPlidarDriver_isConnected(self)

    def reset(self, *args):
        return _rplidar.RPlidarDriver_reset(self, *args)

    def clearNetSerialRxCache(self):
        return _rplidar.RPlidarDriver_clearNetSerialRxCache(self)

    def getAllSupportedScanModes(self, *args):
        return _rplidar.RPlidarDriver_getAllSupportedScanModes(self, *args)

    def getTypicalScanMode(self, *args):
        return _rplidar.RPlidarDriver_getTypicalScanMode(self, *args)

    def startScan(self, force, useTypicalScan, options=0, outUsedScanMode=None):
        return _rplidar.RPlidarDriver_startScan(self, force, useTypicalScan, options, outUsedScanMode)

    def startScanExpress(self, *args):
        return _rplidar.RPlidarDriver_startScanExpress(self, *args)

    def getHealth(self, *args):
        return _rplidar.RPlidarDriver_getHealth(self, *args)

    def getDeviceInfo(self, *args):
        return _rplidar.RPlidarDriver_getDeviceInfo(self, *args)

    def getSampleDuration_uS(self, *args):
        return _rplidar.RPlidarDriver_getSampleDuration_uS(self, *args)

    def setMotorPWM(self, pwm):
        return _rplidar.RPlidarDriver_setMotorPWM(self, pwm)

    def setLidarSpinSpeed(self, *args):
        return _rplidar.RPlidarDriver_setLidarSpinSpeed(self, *args)

    def startMotor(self):
        return _rplidar.RPlidarDriver_startMotor(self)

    def stopMotor(self):
        return _rplidar.RPlidarDriver_stopMotor(self)

    def checkMotorCtrlSupport(self, *args):
        return _rplidar.RPlidarDriver_checkMotorCtrlSupport(self, *args)

    def checkIfTofLidar(self, *args):
        return _rplidar.RPlidarDriver_checkIfTofLidar(self, *args)

    def getFrequency(self, *args):
        return _rplidar.RPlidarDriver_getFrequency(self, *args)

    def startScanNormal(self, *args):
        return _rplidar.RPlidarDriver_startScanNormal(self, *args)

    def checkExpressScanSupported(self, *args):
        return _rplidar.RPlidarDriver_checkExpressScanSupported(self, *args)

    def stop(self, *args):
        return _rplidar.RPlidarDriver_stop(self, *args)

    def grabScanData(self, *args):
        return _rplidar.RPlidarDriver_grabScanData(self, *args)

    def grabScanDataHq(self, *args):
        return _rplidar.RPlidarDriver_grabScanDataHq(self, *args)

    def ascendScanData(self, *args):
        return _rplidar.RPlidarDriver_ascendScanData(self, *args)

    def getScanDataWithInterval(self, nodebuffer, count):
        return _rplidar.RPlidarDriver_getScanDataWithInterval(self, nodebuffer, count)

    def getScanDataWithIntervalHq(self, nodebuffer, count):
        return _rplidar.RPlidarDriver_getScanDataWithIntervalHq(self, nodebuffer, count)
    __swig_destroy__ = _rplidar.delete_RPlidarDriver
    __del__ = lambda self: None
    __swig_setmethods__["_chanDev"] = _rplidar.RPlidarDriver__chanDev_set
    __swig_getmethods__["_chanDev"] = _rplidar.RPlidarDriver__chanDev_get
    if _newclass:
        _chanDev = _swig_property(_rplidar.RPlidarDriver__chanDev_get, _rplidar.RPlidarDriver__chanDev_set)
RPlidarDriver_swigregister = _rplidar.RPlidarDriver_swigregister
RPlidarDriver_swigregister(RPlidarDriver)

def RPlidarDriver_CreateDriver(*args):
    return _rplidar.RPlidarDriver_CreateDriver(*args)
RPlidarDriver_CreateDriver = _rplidar.RPlidarDriver_CreateDriver

def RPlidarDriver_DisposeDriver(drv):
    return _rplidar.RPlidarDriver_DisposeDriver(drv)
RPlidarDriver_DisposeDriver = _rplidar.RPlidarDriver_DisposeDriver

class lidar_sample(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, lidar_sample, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, lidar_sample, name)
    __repr__ = _swig_repr
    __swig_setmethods__["angle"] = _rplidar.lidar_sample_angle_set
    __swig_getmethods__["angle"] = _rplidar.lidar_sample_angle_get
    if _newclass:
        angle = _swig_property(_rplidar.lidar_sample_angle_get, _rplidar.lidar_sample_angle_set)
    __swig_setmethods__["distance"] = _rplidar.lidar_sample_distance_set
    __swig_getmethods__["distance"] = _rplidar.lidar_sample_distance_get
    if _newclass:
        distance = _swig_property(_rplidar.lidar_sample_distance_get, _rplidar.lidar_sample_distance_set)
    __swig_setmethods__["quality"] = _rplidar.lidar_sample_quality_set
    __swig_getmethods__["quality"] = _rplidar.lidar_sample_quality_get
    if _newclass:
        quality = _swig_property(_rplidar.lidar_sample_quality_get, _rplidar.lidar_sample_quality_set)

    def __init__(self):
        this = _rplidar.new_lidar_sample()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _rplidar.delete_lidar_sample
    __del__ = lambda self: None
lidar_sample_swigregister = _rplidar.lidar_sample_swigregister
lidar_sample_swigregister(lidar_sample)

class Rplidar(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Rplidar, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Rplidar, name)
    __repr__ = _swig_repr

    def __init__(self):
        this = _rplidar.new_Rplidar()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _rplidar.delete_Rplidar
    __del__ = lambda self: None

    def connect(self, *args):
        return _rplidar.Rplidar_connect(self, *args)

    def destroy(self):
        return _rplidar.Rplidar_destroy(self)

    def checkHealth(self):
        return _rplidar.Rplidar_checkHealth(self)

    def startMotor(self, scanmode=2):
        if self.checkHealth():
            return _rplidar.Rplidar_startMotor(self, scanmode)
        else:
            raise(RuntimeError("LiDAR is not connected."))

    def stopMotor(self):
        if self.checkHealth():
            return _rplidar.Rplidar_stopMotor(self)
        else:
            raise(RuntimeError("LiDAR is not connected."))

    def getSamples(self, filter_quality=True):
        return _rplidar.Rplidar_getSamples(self, filter_quality)

    def getVectors(self, filter_quality=True):
        return np.array(_rplidar.Rplidar_getVectors(self, filter_quality))

    def getXY(self, filter_quality=True):
        return np.array(_rplidar.Rplidar_getXY(self, filter_quality))[:,::-1]

    def getMap(self, size=(300,300), limit_distance=12000,  filter_quality=True):
        pixmap=np.zeros(size[::-1])

        width,height=(None,None)

        if type(size) in (tuple, list):
            width,height=(size[0]/2,size[1]/2)
        elif type(size) in (str, int, float):
            size=int(size)
            width,height=(size/2,size/2)
        else:
            raise(ValueError("The argument 'size' is not available."))

        coords=np.array(self.getXY())

        is_ranged=np.mean(np.abs(coords)<limit_distance, axis=1)==1.0

        for i in range(len(is_ranged)):
            if is_ranged[i]:
                x,y=( (coords[i]*(1.,-1.)) /limit_distance) * (width,height) + (width,height)
                pixmap[int(y),int(x)]+=1

        std=np.mean(pixmap)+np.std(pixmap)

        if std != 0:
            pixmap=np.where(pixmap>std, std, pixmap)
            pixmap=(pixmap/std)*255

        return pixmap


Rplidar_swigregister = _rplidar.Rplidar_swigregister
Rplidar_swigregister(Rplidar)

class vector_double(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, vector_double, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, vector_double, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _rplidar.vector_double_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _rplidar.vector_double___nonzero__(self)

    def __bool__(self):
        return _rplidar.vector_double___bool__(self)

    def __len__(self):
        return _rplidar.vector_double___len__(self)

    def __getslice__(self, i, j):
        return _rplidar.vector_double___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _rplidar.vector_double___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _rplidar.vector_double___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _rplidar.vector_double___delitem__(self, *args)

    def __getitem__(self, *args):
        return _rplidar.vector_double___getitem__(self, *args)

    def __setitem__(self, *args):
        return _rplidar.vector_double___setitem__(self, *args)

    def pop(self):
        return _rplidar.vector_double_pop(self)

    def append(self, x):
        return _rplidar.vector_double_append(self, x)

    def empty(self):
        return _rplidar.vector_double_empty(self)

    def size(self):
        return _rplidar.vector_double_size(self)

    def swap(self, v):
        return _rplidar.vector_double_swap(self, v)

    def begin(self):
        return _rplidar.vector_double_begin(self)

    def end(self):
        return _rplidar.vector_double_end(self)

    def rbegin(self):
        return _rplidar.vector_double_rbegin(self)

    def rend(self):
        return _rplidar.vector_double_rend(self)

    def clear(self):
        return _rplidar.vector_double_clear(self)

    def get_allocator(self):
        return _rplidar.vector_double_get_allocator(self)

    def pop_back(self):
        return _rplidar.vector_double_pop_back(self)

    def erase(self, *args):
        return _rplidar.vector_double_erase(self, *args)

    def __init__(self, *args):
        this = _rplidar.new_vector_double(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    def push_back(self, x):
        return _rplidar.vector_double_push_back(self, x)

    def front(self):
        return _rplidar.vector_double_front(self)

    def back(self):
        return _rplidar.vector_double_back(self)

    def assign(self, n, x):
        return _rplidar.vector_double_assign(self, n, x)

    def resize(self, *args):
        return _rplidar.vector_double_resize(self, *args)

    def insert(self, *args):
        return _rplidar.vector_double_insert(self, *args)

    def reserve(self, n):
        return _rplidar.vector_double_reserve(self, n)

    def capacity(self):
        return _rplidar.vector_double_capacity(self)
    __swig_destroy__ = _rplidar.delete_vector_double
    __del__ = lambda self: None
vector_double_swigregister = _rplidar.vector_double_swigregister
vector_double_swigregister(vector_double)

class vector_lidar_sample(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, vector_lidar_sample, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, vector_lidar_sample, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _rplidar.vector_lidar_sample_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _rplidar.vector_lidar_sample___nonzero__(self)

    def __bool__(self):
        return _rplidar.vector_lidar_sample___bool__(self)

    def __len__(self):
        return _rplidar.vector_lidar_sample___len__(self)

    def __getslice__(self, i, j):
        return _rplidar.vector_lidar_sample___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _rplidar.vector_lidar_sample___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _rplidar.vector_lidar_sample___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _rplidar.vector_lidar_sample___delitem__(self, *args)

    def __getitem__(self, *args):
        return _rplidar.vector_lidar_sample___getitem__(self, *args)

    def __setitem__(self, *args):
        return _rplidar.vector_lidar_sample___setitem__(self, *args)

    def pop(self):
        return _rplidar.vector_lidar_sample_pop(self)

    def append(self, x):
        return _rplidar.vector_lidar_sample_append(self, x)

    def empty(self):
        return _rplidar.vector_lidar_sample_empty(self)

    def size(self):
        return _rplidar.vector_lidar_sample_size(self)

    def swap(self, v):
        return _rplidar.vector_lidar_sample_swap(self, v)

    def begin(self):
        return _rplidar.vector_lidar_sample_begin(self)

    def end(self):
        return _rplidar.vector_lidar_sample_end(self)

    def rbegin(self):
        return _rplidar.vector_lidar_sample_rbegin(self)

    def rend(self):
        return _rplidar.vector_lidar_sample_rend(self)

    def clear(self):
        return _rplidar.vector_lidar_sample_clear(self)

    def get_allocator(self):
        return _rplidar.vector_lidar_sample_get_allocator(self)

    def pop_back(self):
        return _rplidar.vector_lidar_sample_pop_back(self)

    def erase(self, *args):
        return _rplidar.vector_lidar_sample_erase(self, *args)

    def __init__(self, *args):
        this = _rplidar.new_vector_lidar_sample(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    def push_back(self, x):
        return _rplidar.vector_lidar_sample_push_back(self, x)

    def front(self):
        return _rplidar.vector_lidar_sample_front(self)

    def back(self):
        return _rplidar.vector_lidar_sample_back(self)

    def assign(self, n, x):
        return _rplidar.vector_lidar_sample_assign(self, n, x)

    def resize(self, *args):
        return _rplidar.vector_lidar_sample_resize(self, *args)

    def insert(self, *args):
        return _rplidar.vector_lidar_sample_insert(self, *args)

    def reserve(self, n):
        return _rplidar.vector_lidar_sample_reserve(self, n)

    def capacity(self):
        return _rplidar.vector_lidar_sample_capacity(self)
    __swig_destroy__ = _rplidar.delete_vector_lidar_sample
    __del__ = lambda self: None
vector_lidar_sample_swigregister = _rplidar.vector_lidar_sample_swigregister
vector_lidar_sample_swigregister(vector_lidar_sample)

class vector_vector_double(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, vector_vector_double, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, vector_vector_double, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _rplidar.vector_vector_double_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _rplidar.vector_vector_double___nonzero__(self)

    def __bool__(self):
        return _rplidar.vector_vector_double___bool__(self)

    def __len__(self):
        return _rplidar.vector_vector_double___len__(self)

    def __getslice__(self, i, j):
        return _rplidar.vector_vector_double___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _rplidar.vector_vector_double___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _rplidar.vector_vector_double___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _rplidar.vector_vector_double___delitem__(self, *args)

    def __getitem__(self, *args):
        return _rplidar.vector_vector_double___getitem__(self, *args)

    def __setitem__(self, *args):
        return _rplidar.vector_vector_double___setitem__(self, *args)

    def pop(self):
        return _rplidar.vector_vector_double_pop(self)

    def append(self, x):
        return _rplidar.vector_vector_double_append(self, x)

    def empty(self):
        return _rplidar.vector_vector_double_empty(self)

    def size(self):
        return _rplidar.vector_vector_double_size(self)

    def swap(self, v):
        return _rplidar.vector_vector_double_swap(self, v)

    def begin(self):
        return _rplidar.vector_vector_double_begin(self)

    def end(self):
        return _rplidar.vector_vector_double_end(self)

    def rbegin(self):
        return _rplidar.vector_vector_double_rbegin(self)

    def rend(self):
        return _rplidar.vector_vector_double_rend(self)

    def clear(self):
        return _rplidar.vector_vector_double_clear(self)

    def get_allocator(self):
        return _rplidar.vector_vector_double_get_allocator(self)

    def pop_back(self):
        return _rplidar.vector_vector_double_pop_back(self)

    def erase(self, *args):
        return _rplidar.vector_vector_double_erase(self, *args)

    def __init__(self, *args):
        this = _rplidar.new_vector_vector_double(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    def push_back(self, x):
        return _rplidar.vector_vector_double_push_back(self, x)

    def front(self):
        return _rplidar.vector_vector_double_front(self)

    def back(self):
        return _rplidar.vector_vector_double_back(self)

    def assign(self, n, x):
        return _rplidar.vector_vector_double_assign(self, n, x)

    def resize(self, *args):
        return _rplidar.vector_vector_double_resize(self, *args)

    def insert(self, *args):
        return _rplidar.vector_vector_double_insert(self, *args)

    def reserve(self, n):
        return _rplidar.vector_vector_double_reserve(self, n)

    def capacity(self):
        return _rplidar.vector_vector_double_capacity(self)
    __swig_destroy__ = _rplidar.delete_vector_vector_double
    __del__ = lambda self: None
vector_vector_double_swigregister = _rplidar.vector_vector_double_swigregister
vector_vector_double_swigregister(vector_vector_double)

# This file is compatible with both classic and new-style classes.





from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='pop-autocar3', 
    version='1.0.0',
    author="hanback-lab",
    author_email="lab@hanback.co.kr",
    description="AIoT AutoCar3 library for pop",
    install_requires=[
        "pop-genlib",
        "traitlets",
        "pyaudio",
        "librosa",
        "ipywidgets",
        "opencv-python"
    ],
    long_description=long_description,
    python_requires='>=3.6',
    long_description_content_type="text/markdown",
    packages= find_packages(exclude = ['docs', '__pycache__/']),
    include_package_data=True,   
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import importlib
import builtins as __builtin__

try:
    pkg = __name__.rpartition('.')[0]
    mname = '.'.join((pkg, '_lidar2d')).lstrip('.')
    _lidar2d = importlib.import_module(mname)
except ImportError:
    _lidar2d = importlib.import_module('_lidar2d')


def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError("'%s' object has no attribute '%s'" % (class_type.__name__, name))


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except __builtin__.Exception:
    class _object:
        pass
    _newclass = 0


class SwigPyIterator(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SwigPyIterator, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SwigPyIterator, name)

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _lidar2d.delete_SwigPyIterator
    __del__ = lambda self: None

    def value(self):
        return _lidar2d.SwigPyIterator_value(self)

    def incr(self, n=1):
        return _lidar2d.SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        return _lidar2d.SwigPyIterator_decr(self, n)

    def distance(self, x):
        return _lidar2d.SwigPyIterator_distance(self, x)

    def equal(self, x):
        return _lidar2d.SwigPyIterator_equal(self, x)

    def copy(self):
        return _lidar2d.SwigPyIterator_copy(self)

    def next(self):
        return _lidar2d.SwigPyIterator_next(self)

    def __next__(self):
        return _lidar2d.SwigPyIterator___next__(self)

    def previous(self):
        return _lidar2d.SwigPyIterator_previous(self)

    def advance(self, n):
        return _lidar2d.SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        return _lidar2d.SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        return _lidar2d.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        return _lidar2d.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        return _lidar2d.SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        return _lidar2d.SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        return _lidar2d.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self
SwigPyIterator_swigregister = _lidar2d.SwigPyIterator_swigregister
SwigPyIterator_swigregister(SwigPyIterator)


class VecDouble(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, VecDouble, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, VecDouble, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _lidar2d.VecDouble_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _lidar2d.VecDouble___nonzero__(self)

    def __bool__(self):
        return _lidar2d.VecDouble___bool__(self)

    def __len__(self):
        return _lidar2d.VecDouble___len__(self)

    def __getslice__(self, i, j):
        return _lidar2d.VecDouble___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _lidar2d.VecDouble___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _lidar2d.VecDouble___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _lidar2d.VecDouble___delitem__(self, *args)

    def __getitem__(self, *args):
        return _lidar2d.VecDouble___getitem__(self, *args)

    def __setitem__(self, *args):
        return _lidar2d.VecDouble___setitem__(self, *args)

    def pop(self):
        return _lidar2d.VecDouble_pop(self)

    def append(self, x):
        return _lidar2d.VecDouble_append(self, x)

    def empty(self):
        return _lidar2d.VecDouble_empty(self)

    def size(self):
        return _lidar2d.VecDouble_size(self)

    def swap(self, v):
        return _lidar2d.VecDouble_swap(self, v)

    def begin(self):
        return _lidar2d.VecDouble_begin(self)

    def end(self):
        return _lidar2d.VecDouble_end(self)

    def rbegin(self):
        return _lidar2d.VecDouble_rbegin(self)

    def rend(self):
        return _lidar2d.VecDouble_rend(self)

    def clear(self):
        return _lidar2d.VecDouble_clear(self)

    def get_allocator(self):
        return _lidar2d.VecDouble_get_allocator(self)

    def pop_back(self):
        return _lidar2d.VecDouble_pop_back(self)

    def erase(self, *args):
        return _lidar2d.VecDouble_erase(self, *args)

    def __init__(self, *args):
        this = _lidar2d.new_VecDouble(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    def push_back(self, x):
        return _lidar2d.VecDouble_push_back(self, x)

    def front(self):
        return _lidar2d.VecDouble_front(self)

    def back(self):
        return _lidar2d.VecDouble_back(self)

    def assign(self, n, x):
        return _lidar2d.VecDouble_assign(self, n, x)

    def resize(self, *args):
        return _lidar2d.VecDouble_resize(self, *args)

    def insert(self, *args):
        return _lidar2d.VecDouble_insert(self, *args)

    def reserve(self, n):
        return _lidar2d.VecDouble_reserve(self, n)

    def capacity(self):
        return _lidar2d.VecDouble_capacity(self)
    __swig_destroy__ = _lidar2d.delete_VecDouble
    __del__ = lambda self: None
VecDouble_swigregister = _lidar2d.VecDouble_swigregister
VecDouble_swigregister(VecDouble)

class VecVecDouble(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, VecVecDouble, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, VecVecDouble, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _lidar2d.VecVecDouble_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _lidar2d.VecVecDouble___nonzero__(self)

    def __bool__(self):
        return _lidar2d.VecVecDouble___bool__(self)

    def __len__(self):
        return _lidar2d.VecVecDouble___len__(self)

    def __getslice__(self, i, j):
        return _lidar2d.VecVecDouble___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _lidar2d.VecVecDouble___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _lidar2d.VecVecDouble___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _lidar2d.VecVecDouble___delitem__(self, *args)

    def __getitem__(self, *args):
        return _lidar2d.VecVecDouble___getitem__(self, *args)

    def __setitem__(self, *args):
        return _lidar2d.VecVecDouble___setitem__(self, *args)

    def pop(self):
        return _lidar2d.VecVecDouble_pop(self)

    def append(self, x):
        return _lidar2d.VecVecDouble_append(self, x)

    def empty(self):
        return _lidar2d.VecVecDouble_empty(self)

    def size(self):
        return _lidar2d.VecVecDouble_size(self)

    def swap(self, v):
        return _lidar2d.VecVecDouble_swap(self, v)

    def begin(self):
        return _lidar2d.VecVecDouble_begin(self)

    def end(self):
        return _lidar2d.VecVecDouble_end(self)

    def rbegin(self):
        return _lidar2d.VecVecDouble_rbegin(self)

    def rend(self):
        return _lidar2d.VecVecDouble_rend(self)

    def clear(self):
        return _lidar2d.VecVecDouble_clear(self)

    def get_allocator(self):
        return _lidar2d.VecVecDouble_get_allocator(self)

    def pop_back(self):
        return _lidar2d.VecVecDouble_pop_back(self)

    def erase(self, *args):
        return _lidar2d.VecVecDouble_erase(self, *args)

    def __init__(self, *args):
        this = _lidar2d.new_VecVecDouble(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    def push_back(self, x):
        return _lidar2d.VecVecDouble_push_back(self, x)

    def front(self):
        return _lidar2d.VecVecDouble_front(self)

    def back(self):
        return _lidar2d.VecVecDouble_back(self)

    def assign(self, n, x):
        return _lidar2d.VecVecDouble_assign(self, n, x)

    def resize(self, *args):
        return _lidar2d.VecVecDouble_resize(self, *args)

    def insert(self, *args):
        return _lidar2d.VecVecDouble_insert(self, *args)

    def reserve(self, n):
        return _lidar2d.VecVecDouble_reserve(self, n)

    def capacity(self):
        return _lidar2d.VecVecDouble_capacity(self)
    __swig_destroy__ = _lidar2d.delete_VecVecDouble
    __del__ = lambda self: None
VecVecDouble_swigregister = _lidar2d.VecVecDouble_swigregister
VecVecDouble_swigregister(VecVecDouble)


from .CAN import Can
import threading


class Thermopile:
    ACT_THERMO = 0x2A0
    BRD_THERMO = 0x3A0

    def __init__(self, dev="can0", bitrate=500000):
        self.__can = Can(dev, bitrate)
        self.__func = None
        self.__param = None
        self.__thread = None
        self.__stop = False
        self.__can.setFilter(Thermopile.BRD_THERMO)

    def __del__(self):
        self.stop()

    def __callback(self):
        while not self.__stop:
            id, dlc, payload = self.__can.read(timeout=0.1)
            if not payload:
                continue
            elif id == Thermopile.BRD_THERMO:
                ir = (payload[1] << 8) | payload[0]
                if self.__param:
                    self.__func((ir), self.__param)
                else:
                    self.__func((ir))

    def read(self):
        try:
            self.__can.write(Thermopile.ACT_THERMO, [0x00])
            id, dlc, payload = self.__can.read(timeout=2.0)

            if id == Thermopile.BRD_THERMO:
                ir = (payload[1] << 8) | payload[0]

            return ir
        except:
            return None

    def callback(self, func, repeat=1, param=None):
        if not self.__thread:
            self.__func = func
            self.__param = param
            self.__stop = False
            self.__thread = threading.Thread(target=self.__callback)
            self.__thread.start()
            self.__can.write(Thermopile.ACT_THERMO, [repeat & 0xFF])

    def stop(self):
        if self.__thread:
            self.__stop = True
            self.__thread = None
            self.__can.write(Thermopile.ACT_THERMO, [0x00])



import can
import time


class Can:
    def __init__(self, dev="can0", bitrate=500000, fd=True):
        self._bus = can.interface.Bus(
            channel=dev, bustype="socketcan", bitrate=bitrate, fd=True
        )
        self.__filter = []

    def __del__(self):
        self._bus.shutdown()

    def write(self, msg_id, buf, is_extended=False):
        msg = can.Message(arbitration_id=msg_id, is_extended_id=is_extended, data=buf)
        try:
            self._bus.send(msg)
            time.sleep(0.005)
            # time.sleep(0.020)
        except can.CanError:
            print("Can Interface Message does not Send")

    def read(self, timeout=2):
        msg = self._bus.recv(timeout=timeout)
        if msg:
            return msg.arbitration_id, msg.dlc, list(msg.data)
        else:
            return (None, None, None)

    def setFilter(self, can_id, mask=0x7FF):
        for i, f in enumerate(self.__filter):
            if f["can_id"] == can_id:
                if mask == 0:
                    del self.__filter[i]
                    return
                else:
                    self.__filter[i]["can_mask"] = mask
                    break
        else:
            self.__filter.append({"can_id": can_id, "can_mask": mask})
        self._bus.set_filters(self.__filter)
